using GLMakie
using Printf
GLMakie.activate!()
using dg1d


MT = GLMakie.Makie.MathTeXEngine
mt_fonts_dir = joinpath(dirname(pathof(MT)), "..", "assets", "fonts", "NewComputerModern")

mytheme = Theme(
  fonts = ( regular = joinpath(mt_fonts_dir, "NewCM10-Regular.otf"),
            bold = joinpath(mt_fonts_dir, "NewCM10-Bold.otf"),
            bold_italic = joinpath(mt_fonts_dir, "NewCM10-BoldItalic.otf"),
            italic = joinpath(mt_fonts_dir, "NewCM10-Italic.otf"),
  ),
  fontsize=30
)


function read_data(dir::String)
  output = Output(joinpath(dir, "output1d.h5"))
  variables = output.variables
  varnames = [ last(split(var, ".")) for var in variables ]
  nvars = length(variables)
  ts = output.times
  Nts = length(ts)
  u(t) = read_variable(output, "u", t)
  return ts, output.x, u
end

function make_gif(filename, ts, x, u, N, K)
  framerate=10
  buf_x = zeros((N+2)*K)
  buf_y = zeros((N+2)*K)
  # separate cell data by NaN rows to make lines discontinuous
  fill!(buf_x, NaN); fill!(buf_y, NaN)
  # idxs = [ i for i in 1:length(buf_x) if i % (N+2) != 0 ]
  idxs = [ i%(N+2)!=0 for i in 1:length(buf_x) ]
  buf_x[idxs] .= x
  buf_y[idxs] .= u(0.0)
  obs_y = Observable(buf_y)
  obs_t = Observable(ts[1])
  lbl_t = lift(obs_t) do t
    fmt_t = @sprintf "%.02f" t
    L"t = %$fmt_t"
  end
  with_theme(mytheme) do
    fig = Figure()
    ax = Axis(fig[1,1], xlabel=L"x", ylabel="u")
    lines!(ax, buf_x, obs_y, label=L"K=%$K")
    text!(ax, -1.0, 0.5; text=lbl_t)
    axislegend(ax)
    record(fig, joinpath(@__DIR__,filename), ts[2:end]; framerate) do t
      obs_y.val[idxs] .= u(t); notify(obs_y)
      obs_t[] = t
    end
  end
end


let


dir = joinpath(@__DIR__, "heateq_sine_1")
ts, x, u = read_data(dir)
N, K = 1, 10
make_gif("heateq_sine_1.gif", ts, x, u, N, K)

dir = joinpath(@__DIR__, "heateq_sine_2")
ts, x, u = read_data(dir)
N, K = 1, 40
make_gif("heateq_sine_2.gif", ts, x, u, N, K)

dir = joinpath(@__DIR__, "heateq_sine_3")
ts, x, u = read_data(dir)
N, K = 1, 80
make_gif("heateq_sine_3.gif", ts, x, u, N, K)

# analytical solution
L = 4.0
x = range(extrema(x)..., length=500)
u(t, x) = exp(-4*pi^2/L^2 * t) * sin(2*pi*x/L)
framerate=10
obs_y = Observable(u.(0.0,x))
obs_t = Observable(ts[1])
lbl_t = lift(obs_t) do t
  fmt_t = @sprintf "%.02f" t
  L"t = %$fmt_t"
end
with_theme(mytheme) do
  fig = Figure()
  ax = Axis(fig[1,1], xlabel=L"x", ylabel="u")
  lines!(ax, x, obs_y, label="exact")
  text!(ax, -1.0, 0.5; text=lbl_t)
  axislegend(ax)
  record(fig, joinpath(@__DIR__,"heateq_sine_exact.gif"), ts[2:end]; framerate) do t
    obs_y[] = u.(t,x)
    obs_t[] = t
  end
end


end
