#######################################################################
#                              Timestep                               #
#######################################################################

function timestep(env, P::Project, ::Mesh1d)
  @unpack cache, mesh = env
  @unpack equation = P
  @unpack v = get_static_variables(cache)

  broadcast_volume!(maxspeed, equation, mesh)
  vmax = dg1d.absolute_maximum(v)
  vmax_limit = 1e4
  if vmax > vmax_limit
    @warn "Limiting timestep due to maximum speed exceeding $vmax_limit"
    vmax = vmax_limit
  end

  @unpack element = mesh
  @unpack N, z = element
  dl = min_grid_spacing(mesh)

  dt = dl^2 / (4*vmax)
  return dt
end


#######################################################################
#                           RHS computation                           #
#######################################################################


function rhs!(env, P, mesh::Mesh1d)

  @unpack cache, mesh = env
  @unpack equation = P

  @unpack flx_u, flx_q, q = get_static_variables(cache)
  @unpack u               = get_dynamic_variables(cache)
  @unpack rhs_u           = get_rhs_variables(cache)
  @unpack nflx_u, nflx_q  = get_bdry_variables(cache)
  @unpack bdry_u, bdry_q  = get_bdry_variables(mesh.cache)

  ## solve auxiliary equation: q - ∂x u = 0
  dg1d.interpolate_face_data!(mesh, u, bdry_u)
  broadcast_volume!(ldg_flux, equation, mesh)
  broadcast_faces!(ldg_nflux, equation, mesh)
  compute_rhs_weak_form!(q, flx_q, nflx_q, mesh)

  ## compute rhs of regularized equation: ∂t u - ∂x q = 0
  dg1d.interpolate_face_data!(mesh, q, bdry_q)
  broadcast_volume!(flux, equation, mesh)
  broadcast_faces!(nflux, equation, mesh)

  compute_rhs_weak_form!(rhs_u, flx_u, nflx_u, mesh)

  return
end
